unit untTopicPostThread;

interface

uses
  Classes, SysUtils, Dialogs, DzURL,
  untStreamTool, untTool,
  untBoard, untOnlineBoard, untGlobal, untTopic;

type
  TBoardStateType = (bsTopicReceived, bsWriteDone);

  TTopicPostThread = class(TThread)
  protected
    FTopic: TTopic;
    FParent         : TOnlineBoard;
    FTitle          : string;
    FPostName       : string;
    FPostEmail      : string;
    FPostBody       : string;
    FErrorString    : string;
    FErrorCode      : TFrameworkErrorType;
    FBoardStateType : TBoardStateType;
    procedure RaiseBBSError();
    procedure BBS_Complete(Sender : TObject);
    procedure BBS_Error(Sender: TObject;
                        const ErrorCode: TFrameworkErrorType;
                        const ErrorString: string);
    procedure BBS_StatusTextChange(Sender: TObject; const StatusText: string);
  public
    OnComplete: TNotifyEvent;
    OnStatusTextChange: TStatusTextChangeEvent;
    OnError: TFrameworkError;
    procedure   Execute; override;
    constructor Create(Parent: TOnlineBoard; Topic: TTopic; const Title, PostName, PostEmail, PostBody: string);
  end;

implementation

{ TTopicPostThread }

constructor TTopicPostThread.Create(Parent: TOnlineBoard; Topic: TTopic; const Title, PostName, PostEmail, PostBody: string);
begin
  inherited Create(true);

  FTopic := Topic;
  FTopic.OnStatusTextChange := BBS_StatusTextChange;
  FTopic.OnComplete         := BBS_Complete;
  FTopic.OnError            := BBS_Error;

  FParent := Parent;
  FPostName  := PostName;
  FPostEmail := PostEmail;
  FPostBody  := PostBody;
  FTitle     := Title;
end;

procedure TTopicPostThread.Execute;
begin
  inherited;

  // 
  FTopic.Post(FPostName, FPostEmail, FPostBody);

  Terminate;
end;

procedure TTopicPostThread.BBS_Complete(Sender: TObject);
begin
  if Assigned(OnComplete) then
  begin
    OnComplete(self);
  end;
end;

procedure TTopicPostThread.BBS_StatusTextChange(Sender: TObject;
                                                const StatusText: string);
begin
  if Assigned(OnStatusTextChange) then
  begin
    OnStatusTextChange(self, StatusText);
  end;
end;

procedure TTopicPostThread.BBS_Error(Sender: TObject;
                                     const ErrorCode: TFrameworkErrorType;
                                     const ErrorString: string);
begin
  FErrorString := ErrorString;
  FErrorCode   := ErrorCode;
  Synchronize(RaiseBBSError);
end;

procedure TTopicPostThread.RaiseBBSError;
begin
  if Assigned(OnError) then
  begin
    OnError(self, FErrorCode, FErrorString);
  end;
end;

end.
